#include <stdint.h>
#include <SFML/System.h>
#include <SFML/Graphics.h>
#include <SFML/Window.h>

extern float displayScale;
extern int clocksteps;
extern uint8_t halt;
extern uint8_t showDebug;
extern uint8_t showHelp;

extern void resetSystem();
extern void irq6502();
extern void nmi6502();
extern void step6502();
extern void updateRenderStates();
extern void scanKeyboard();

static sfKeyCode currentKey = sfKeyUnknown;

void pollEvents(sfRenderWindow *window){

	sfEvent event;
	while (sfRenderWindow_pollEvent(window, &event)) {
		switch (event.type){
			case sfEvtClosed:
				sfRenderWindow_close(window);
				break;
				
			case sfEvtResized:
				sfRenderWindow_setView(window, sfView_createFromRect((sfFloatRect){0, 0, event.size.width, event.size.height}));
				break;
			case sfEvtKeyPressed:
				switch (event.key.code){
					case sfKeyF1:	/* Close Window or show help*/
						sfRenderWindow_close(window);
						break;
						
					case sfKeyF2:	/* Show Key Help */
						showHelp = !showHelp;
						break;
						
					case sfKeyF3:	/* Show Debug Menu */
						showDebug = !showDebug;
						break;
						
					case sfKeyF4:	/* CPU Halt */
						halt = !halt;
						break;
						
					case sfKeyF5:	/* CPU Reset */
						resetSystem();
						break;
						
					case sfKeyF6:	/* Scale -- */
						if (displayscale > 1) displayscale--;
						updateRenderStates();
						break;
					
					case sfKeyF7:	/* Scale ++ */
						displayscale++;
						updateRenderStates();
						break;
						
					case sfKeyF8:	/* CPU Singlestep */
						for (int i = 0; i < clocksteps; i++) step6502();
						break;
					
					case sfKeyF9:	/* CPU Save Snapshot */
						break;
						
					case sfKeyF10:	/* CPU Load recent or given snapshot */
						break;
						
					case sfKeyF11:	/* CPU NMI */
						nmi6502();
						break;
						
					case sfKeyF12:	/* CPU IRQ */
						irq6502();
						break;
					
					default:
						currentKey = event.key.code;
						break;
				}
				break;
			case sfEvtKeyReleased:
				// To tell keyboard.c, that no key is being pressed
				currentKey = sfKeyUnknown;
				break;
					
			default: break;
		}
	}
}
